﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Renci.SshNet;     // ソリューションエクスプローラでプロジェクト配下の「参照」を右クリックし、「NuGetパッケージの管理」を開き、「参照」タブを開いて「SSH.NET」を検索しインストール

namespace WindowsFormsApp1
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            textBoxHost.Text = "";
            textBoxPort.Text = "22";
            textBoxUser.Text = "";
            textBoxPassword.Text = "";
        }

        private void buttonRun_Click(object sender, EventArgs e)
        {
            buttonRun.Enabled = false;
            try
            {
                // コネクション情報
                ConnectionInfo info = new ConnectionInfo(
                    textBoxHost.Text,
                    Convert.ToInt32(textBoxPort.Text),
                    textBoxUser.Text,
                    new AuthenticationMethod[] {
                        new PasswordAuthenticationMethod(textBoxUser.Text, textBoxPassword.Text)
                    }
                );

                // SSHクライアントオブジェクトを生成し接続
                SshClient ssh = new SshClient(info);
                ssh.Connect();
                if (!ssh.IsConnected)
                {
                    txtMsg("SSH接続に失敗しました");
                    return;
                }

                // SSHコマンド実行
                var commandString = textBoxCommand.Text;
                SshCommand cmd = ssh.CreateCommand(commandString);
                txtMsg("$ " + commandString);
                cmd.Execute();

                var stdOut = cmd.Result;
                var stdErr = cmd.Error;

                if (stdOut != string.Empty)
                {
                    // コマンド実行結果を表示
                    txtMsg(stdOut);
                }

                if (cmd.ExitStatus != 0 && stdErr != string.Empty)
                {
                    // エラー出力を表示
                    txtMsg(stdErr);
                }

                // 接続終了
                ssh.Disconnect();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }
            textBoxCommand.Text = "";
            buttonRun.Enabled = true;
        }

        private void textBoxCommand_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                buttonRun_Click(null, null);
            }
        }

        private void txtMsg(string msg)
        {
            textBoxMsg.Text += msg.Replace("\n", "\r\n") + "\r\n";
            textBoxMsg.SelectionStart = textBoxMsg.Text.Length;
            textBoxMsg.ScrollToCaret();
        }

        // SCPアップロード
        private void buttonUpload_Click(object sender, EventArgs e)
        {
            buttonUpload.Enabled = false;
            try
            {
                using (var client = new ScpClient(textBoxHost.Text, Convert.ToInt32(textBoxPort.Text), textBoxUser.Text, textBoxPassword.Text))
                {
                    client.RemotePathTransformation = RemotePathTransformation.ShellQuote;
                    client.Connect();

                    System.IO.FileInfo fi = new System.IO.FileInfo(textBoxUpFile.Text);
                    client.Upload(fi, textBoxUpPath.Text);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }
            buttonUpload.Enabled = true;

        }

        // SCPダウンロード
        private void buttonDownload_Click(object sender, EventArgs e)
        {
            buttonDownload.Enabled = false;
            try
            {
                using (var client = new ScpClient(textBoxHost.Text, Convert.ToInt32(textBoxPort.Text), textBoxUser.Text, textBoxPassword.Text))
                {
                    client.RemotePathTransformation = RemotePathTransformation.ShellQuote;
                    client.Connect();

                    System.IO.FileInfo fi = new System.IO.FileInfo(textBoxDownPath.Text);
                    client.Download(textBoxDownFile.Text, fi);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }
            buttonDownload.Enabled = true;
        }
    }
}
